// CDXTileLayer.cpp: implementation of the CDXTileLayer class.
//
//////////////////////////////////////////////////////////////////////

#include "CDXTileEngine.h"
#include "CDXTileLayer.h"
#include "CDXTileStack.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDXTileLayer::CDXTileLayer(CDXTileEngine* pEngine, int mapDimX, int mapDimY, int tileSizeX, int tileSizeY)
{
	Init(pEngine);
	SetMapDimension(mapDimX, mapDimY);
	ClearMap();
	SetTileSize(tileSizeX, tileSizeY);
}

CDXTileLayer::CDXTileLayer(CDXTileEngine* pEngine)
{
	Init(pEngine);
}

CDXTileLayer::~CDXTileLayer()
{
	if (m_TileStacks)
		delete[] m_TileStacks;
	if (m_Sprites)
		delete[] m_Sprites;
}

BOOL CDXTileLayer::LoadFile(char *filename)
{
	// under development
	return true;
}

BOOL CDXTileLayer::SaveFile(char *filename)
{
	// under development
	return true;
}

BOOL CDXTileLayer::Draw()
{
	// Get the active viewer first
	CDXTileViewer* pViewer = GetActiveViewer();
	Assert(pViewer, "CDXTileLayer::Draw\nNo Viewer");
	pViewer->Draw(this);
	return true;
}

void CDXTileLayer::SetTileStack(int x, int y, CDXTileStack *tileStack)
{
	Assert(m_TileStacks, "CDXTileLayer::SetTileStack\nNo Tile Capability");
	Assert(x < m_MapDimX && y < m_MapDimY, "CDXTileLayer::SetTileStack\nCoordinat out of range");
	m_TileStacks[y*m_MapDimX+x] = tileStack;
}

void CDXTileLayer::SetMapDimension(int mapDimX, int mapDimY)
{
	// release old data
	if (m_TileStacks)
		delete[] m_TileStacks;
	if (m_Sprites)
		delete[] m_Sprites;

	m_TileStacks = 0;
	m_Sprites = 0;

	m_MapDimX = mapDimX;
	m_MapDimY = mapDimY;

	BOOL bTileCap = GetCapabilities(CAP_TILE);
	BOOL bSpriteCap = GetCapabilities(CAP_SPRITE);

	// create new
	if (bTileCap)
		m_TileStacks = new CDXTileStack*[m_MapDimY*m_MapDimX];
	if (bSpriteCap)
		m_Sprites = new CPtrArray[m_MapDimY*m_MapDimX];

}

void CDXTileLayer::Init(CDXTileEngine* pEngine)
{
	m_pEngine = pEngine;
	m_pViewer = 0;
	m_MapDimX = 0;
	m_MapDimY = 0;
	m_TileSizeX = 0;
	m_TileSizeY = 0;
	m_TileStacks = 0;
	m_Sprites = 0;
	SetFlag(BIT_LAYERCAP_TILE);
	SetFlag(BIT_LAYERCAP_SPRITE);
	SetVisibleTileBonus(0,0,0,0);
}

void CDXTileLayer::SetTileSize(int tileSizeX, int tileSizeY)
{
	m_TileSizeX = tileSizeX;
	m_TileSizeY = tileSizeY;
}

void CDXTileLayer::ClearMap()
{
	Assert(m_TileStacks, "CDXTileLayer::ClearMap\nNo Tile Capability");
	int count = GetTileStackCount();
	for (int i=0; i<count; i++)
		m_TileStacks[i] = NULL;
}

int CDXTileLayer::GetTileStackCount()
{
	return m_MapDimX*m_MapDimY;
}

CDXTileStack* CDXTileLayer::GetTileStack(int coordX, int coordY)
{
	Assert(m_TileStacks, "CDXTileLayer::GetTileStack\nNo Tile Capability");
	return m_TileStacks[coordY*m_MapDimX+coordX];
}

CDXTileViewer* CDXTileLayer::SetViewer(CDXTileViewer *pViewer)
{
	CDXTileViewer* pOldViewer = m_pViewer;
	m_pViewer = pViewer;
	return pOldViewer;
}

CDXTileViewer* CDXTileLayer::GetViewer()
{
	return m_pViewer;
}

CDXTileViewer* CDXTileLayer::GetActiveViewer()
{
	return GetViewer() ? GetViewer() : GetEngine()->GetViewer();
}

CDXTileEngine* CDXTileLayer::GetEngine()
{
	return m_pEngine;
}


void CDXTileLayer::GetTileSize(int &tileSizeX, int &tileSizeY)
{
	tileSizeX = m_TileSizeX; tileSizeY = m_TileSizeY;
}

void CDXTileLayer::PosToCoord(int posX, int posY, int &coordX, int &coordY)
{
	coordX = posX/m_TileSizeX;
	coordY = posY/m_TileSizeY;
}

void CDXTileLayer::UpdateSprite(CDXTileSpriteBase *pSprite, int oldCoordX, int oldCoordY, int &spriteIndex, int newCoordX, int newCoordY)
{
	Assert(m_Sprites, "CDXTileLayer::UpdateSprite\nNo Sprite Capability");
	// remove old info first
	if (spriteIndex != -1)
		m_Sprites[oldCoordY*m_MapDimX+oldCoordX].RemoveAt(spriteIndex);

	// the new coord out of range, don't add info
	if (newCoordX < 0 || newCoordX >= m_MapDimX ||
		newCoordY < 0 || newCoordY >= m_MapDimY)
	{
		spriteIndex = -1;
		return;
	}

	// add info for new coord
	spriteIndex = 
		m_Sprites[newCoordY*m_MapDimX+newCoordX].Add(pSprite);
}


CPtrArray* CDXTileLayer::GetSpriteArray(int coordX, int coordY)
{
	Assert(m_Sprites, "CDXTileLayer::GetSpriteArray\nNo Sprite Capability");
	return &m_Sprites[coordY*m_MapDimX+coordX];
}

/*void CDXTileLayer::SetCapabilities(BOOL bTileCap, BOOL bSpriteCap)
{
	SetFlag(BIT_LAYERCAP_TILE, bTileCap);
	SetFlag(BIT_LAYERCAP_SPRITE, bSpriteCap);
	
	// This function will destroy all data
	SetMapDimension(m_MapDimX, m_MapDimY);
}

void CDXTileLayer::GetCapabilities(BOOL &bTileCap, BOOL &bSpriteCap)
{
	bTileCap = GetFlag(BIT_LAYERCAP_TILE);
	bSpriteCap = GetFlag(BIT_LAYERCAP_SPRITE);
}
*/
void CDXTileLayer::SetCapabilities(DWORD capabilities)
{
	SetFlag(BIT_LAYERCAP_TILE, capabilities & CAP_TILE);
	SetFlag(BIT_LAYERCAP_SPRITE, capabilities & CAP_SPRITE);

	// This function will destroy all data
	SetMapDimension(m_MapDimX, m_MapDimY);
}

BOOL CDXTileLayer::GetCapabilities(DWORD capabilities)
{
	if (capabilities & CAP_TILE && !GetFlag(BIT_LAYERCAP_TILE))
		return false;
	if (capabilities & CAP_SPRITE && !GetFlag(BIT_LAYERCAP_SPRITE))
		return false;
	return true;
}

void CDXTileLayer::SetCapability(DWORD capability, BOOL set)
{
	if (capability == CAP_TILE)
		SetFlag(BIT_LAYERCAP_TILE, set); else
	if (capability == CAP_SPRITE)
		SetFlag(BIT_LAYERCAP_SPRITE, set);
}

void CDXTileLayer::SetVisibleTileBonus(int leftTile, int topTile, int rightTile, int bottomTile)
{
	m_VisibleTileBonus.left = leftTile;
	m_VisibleTileBonus.top = topTile;
	m_VisibleTileBonus.right = rightTile;
	m_VisibleTileBonus.bottom = bottomTile;
}
